package es.verdnatura.presentation.view.feature.login.model



/*class LoginItemVO(
    val id: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""
)*/

private val s = ""

class LoginSalixVO(
    val user: String = "",
    val password: String = "",
    val token: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class SalixMessageVO(
    val workerId: Int = 0,
    val message: String = "",
    val token: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class SalixGrupo(
    val to: String = "",
    val message: String = ""

)


class versionApp(
    val appname: String = "",
    val version: String = "",
    var IsVersionCritical: Int = 0,
    var url: String="",
    var isError: Boolean = false,
    var errorMessage: String = ""
)

class LoginDevice(
    val vMessage: String? = "",
    val vIsAuthorized: String = "",
    var isError: Boolean = false,
    var errorMessage: String = ""

)

class workerId(
    val Id: String? = "",
    var isError: Boolean = false,
    var errorMessage: String = ""

)